"use client";

import { zodResolver } from "@hookform/resolvers/zod";
import { useForm } from "react-hook-form";
import { z } from "zod";

import { Button } from "@/components/ui/button";
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from "@/components/ui/form";
import { toast } from "@/components/ui/use-toast";
import { Input } from "./ui/input";
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "./ui/select";
import { Account } from "@/types/account";

const displayFormSchema = z.object({
  email: z.string().trim().min(1, { message: "Please select a user." }).email(),
  balance: z.string().trim().min(1, "Balance is required."),
  organization: z
    .string()
    .trim()
    .min(1, { message: "Please select an organization." }),
});

type DisplayFormValues = z.infer<typeof displayFormSchema>;

// This can come from your database or API.
const defaultValues: Partial<DisplayFormValues> = {
  balance: "",
  email: "",
  organization: "",
};

type Props = {
  account?: Account;
  onSubmit: () => void;
  btn1_content: string;
  btn2_content: string;
};

export default function AccountForm({
  account,
  onSubmit: onFormSubmit,
  btn1_content,
  btn2_content,
}: Props) {
  const form = useForm<DisplayFormValues>({
    resolver: zodResolver(displayFormSchema),
    defaultValues: {
      balance: account?.balance ? String(account?.balance) : "",
      email: account?.email || "",
      organization: account?.organization || "",
    },
  });

  function onSubmit(data: DisplayFormValues) {
    toast({
      title: "You submitted the following values:",
      description: (
        <pre className="mt-2 w-[340px] rounded-md bg-slate-950 p-4">
          <code className="text-white">{JSON.stringify(data, null, 2)}</code>
        </pre>
      ),
    });
  }

  return (
    <Form {...form}>
      <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
        <FormField
          control={form.control}
          name="email"
          render={({ field }) => (
            <FormItem>
              <FormLabel>User</FormLabel>
              <Select onValueChange={field.onChange} value={field.value}>
                <FormControl>
                  <SelectTrigger>
                    <SelectValue placeholder="Select a user" />
                  </SelectTrigger>
                </FormControl>
                <SelectContent>
                  <SelectItem value="john@gmail.com">john@gmail.com</SelectItem>
                  <SelectItem value="sarah@example.com">
                    sarah@example.com
                  </SelectItem>
                  <SelectItem value="michael@example.com">
                    michael@example.com
                  </SelectItem>
                </SelectContent>
              </Select>
              <FormMessage />
            </FormItem>
          )}
        />

        <FormField
          control={form.control}
          name="balance"
          render={({ field }) => (
            <FormItem>
              <FormLabel>Balance</FormLabel>
              <FormControl>
                <Input placeholder="Enter balance..." {...field} />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />

        <FormField
          control={form.control}
          name="organization"
          render={({ field }) => (
            <FormItem>
              <FormLabel>Organization</FormLabel>
              <Select onValueChange={field.onChange} defaultValue={field.value}>
                <FormControl>
                  <SelectTrigger>
                    <SelectValue placeholder="Select a organization" />
                  </SelectTrigger>
                </FormControl>
                <SelectContent>
                  <SelectItem value="XYZ Tech Inc.">XYZ Tech Inc.</SelectItem>
                  <SelectItem value="ABC Solutions">ABC Solutions</SelectItem>
                  <SelectItem value="Acme Corporation">
                    Acme Corporation
                  </SelectItem>
                </SelectContent>
              </Select>
              <FormMessage />
            </FormItem>
          )}
        />

        <div className="flex items-center gap-4 flex-col sm:flex-row">
          <Button className="w-full" size="lg">
            {btn1_content}
          </Button>
          <Button variant="secondary" className="w-full" size="lg">
            {btn2_content}
          </Button>
        </div>
      </form>
    </Form>
  );
}
